use std::{fs::File, path::PathBuf, time::Instant};

use binrw::{BinRead, BinWriterExt};
use clap::Parser;
use clap_verbosity_flag::{InfoLevel, Verbosity};
use ezcad::file::EzCadHeader;
use log::{debug, info};

use crate::config::{Config, PatchPens};

mod config;

#[derive(Debug, Parser)]
struct Cli {
    /// Input file to parse
    #[arg(short, long)]
    input: PathBuf,

    /// Output file to write to
    #[arg(short, long)]
    output: Option<PathBuf>,

    /// Configuration file
    #[arg(short, long)]
    config: Option<PathBuf>,

    #[command(flatten)]
    verbose: Verbosity<InfoLevel>,
}

fn main() {
    let cli: Cli = Cli::parse();

    env_logger::builder()
        .format_timestamp(None)
        .filter_level(cli.verbose.log_level_filter())
        .init();

    let mut input: File = File::open(cli.input).unwrap();

    let time: Instant = Instant::now();
    let mut file: EzCadHeader = EzCadHeader::read_le(&mut input).unwrap();
    info!("Decode time: {:?}", time.elapsed());

    // Print info on pens with non-default settings
    for pen in file
        .pens_offset
        .data
        .pens
        .iter()
        .filter(|x| *x.use_default == 0)
    {
        debug!("{:#?}", pen);
    }

    // Print all objects
    for layer in file.layers_offset.iter() {
        for object in layer.objects.iter() {
            debug!("{:#?}", object);
        }
    }

    if let Some(config) = cli.config {
        info!("Processing config file {}", config.to_string_lossy());
        let config: String = std::fs::read_to_string(config).unwrap();
        let config: Config = serde_yaml::from_str(&config).unwrap();
        debug!("{:#?}", config);

        // Patch pen settings
        config.pens.patch(&mut file.pens_offset.data.pens);
    }

    if let Some(output) = cli.output {
        let mut output = File::create(output).unwrap();
        let time: Instant = Instant::now();
        output.write_le(&file).unwrap();
        info!("Encode time: {:?}", time.elapsed());
    }
}
